/*
 * class_reads.h
 *
 *  Created on: Oct 28, 2010
 *      Author: philipp_
 */

#ifndef CLASS_READS_H_
#define CLASS_READS_H_

#include <iostream>
#include <vector>
#include <fstream>

#include "structs.h"
#include "../parser/parameter.h"

using std::streampos;
using std::string;
using std::vector;
using std::cout;
using std::endl;

class Reads {

private:
	char type; //filetzp fasta/fastq
	size_t max_length; //needed because of mem architecture on gpu.
	string filename;
	streampos file_pos;
	int startID;
	int number;
	bool direction;


	void init() {
		direction = true;
		type = ' ';
		max_length = 0;
		file_pos = 0;
		startID = 0;
		number = 0;
	}

public:

	//TODO try to reduce
	vector<string> forward;
	vector<string> backward;
	vector<vector<HashKey> > maped_reads;
	vector<vector<HashKey> > maped_reads_back;
	vector<Matches> matches;


	Reads() {
	}

	Reads(string file) {
		filename.clear();
		filename = file;
		init();
	}

	~Reads() {
		filename.clear();
		clear_keys();
		clear_reads();

		for(vector<Matches>::iterator i= matches.begin();i!=matches.end();i++){
			clear_match(i);
		}
		matches.clear();
	}

	//========== setter: ============
	void set_file_name(string file) {
		filename = file;
		init();
	}

	void set_max_length(int max) {
		max_length = max;
	}

	void set_file_pos(streampos pos) {
		file_pos = pos;
	}
	void set_startID(int ID) {
		startID = ID;
	}

	//start ID is incremented by the amount of reads
	void increment_stratId() {
		startID = startID + forward.size();
	}

	void set_type(char ty) {
		type = ty;
	}
	void set_direction(bool dir) {
		direction = dir;
	}

	//============== getter ==============
	size_t get_max_length() {
		return max_length;
	}

	streampos get_filepos() {
		return file_pos;
	}

	int get_startID() {
		return startID;
	}

	int get_number() {
		return number;
	}
	const char * get_filename() {
		return filename.c_str();
	}

	char get_type() {
		return type;
	}
	bool get_direction() {
		return direction;
	}

	//========== functions: =================
	//brings all the reads to the same size. Needed for the GPU memory
	void fit_size(bool dir) {

		vector<string>::iterator j;
		if (dir) { // if the user wants the backward calculation
			j = backward.begin();
		}

		for (vector<string>::iterator i = forward.begin(); i != forward.end(); i++) {

			if (dir && (*i).size() != (*j).size()) {
				cout << "Error back and front are not the same!" << endl;
			}

			if (max_length > (*i).size()) {

				//enlarge reads if needed:
				for (size_t t = (*i).size(); t < max_length; t++) {
					(*i) += '#'; //fill up character (forward reads)
					if (dir) {
						(*j) += '#';//fill up character (backward reads)
					}
				}
			}
			if (dir) {
				j++;
			}
		}
	}

	//converts k-mers sequences to ints;
	void map_keys(bool dir, Parameters parameters) {
		vector<string>::iterator begin;
		vector<string>::iterator end;

		if (dir) {
			begin = forward.begin();
			end = forward.end();
		} else {
			begin = backward.begin();
			end = backward.end();
		}
		vector<vector<HashKey> > tmp;
		vector<Matches>::iterator t = matches.begin();

		for (vector<string>::iterator i = begin; i != end; i++) {
			vector<HashKey> temp_vec;

			if (parameters.max_equal_scoring>1 || (*t).match[parameters.X_best_hits - 1].score < (*t).orig_length * parameters.match[0]) {
				for (size_t j = 0; j + parameters.wordsize < max_length; j = j + parameters.step_size) {
					HashKey temp;
					//if not A,C,G,T FatsHash returns -1. -> will not be stored
					temp.key= FatsHash2((*i).substr(j,parameters.wordsize));//,parameters);

					if (temp.key >= 0 && temp.key <= parameters.mapsize) {
						temp.pos = j;
						temp_vec.push_back(temp);
					}
				}

			}

			tmp.push_back(temp_vec);
			temp_vec.clear();
			t++;
		}

//		if(parameters.mode==0){
//			BSlookup.clear();
//		}

		if (dir) {
			maped_reads = tmp;
		} else {
			maped_reads_back = tmp;
		}
		for(vector<vector<HashKey> >::iterator t=tmp.begin();t!=tmp.end();t++){
			(*t).clear();
		}
		tmp.clear();
	}





	void inc_numbner() {
		number++;
	}
	void clear_match(vector<Matches>::iterator match){
		(*match).read_name.clear();
		for(vector<Match>::iterator j =(*match).match.begin();j!=(*match).match.end();j++){
			(*j).read.clear();
			(*j).scaff.clear();
		}
		(*match).match.clear();
	}
	void clear_keys() {
		for(vector<vector<HashKey> >::iterator i = maped_reads.begin();i!=maped_reads.end();i++){
			(*i).clear();
		}
		maped_reads.clear();
		for(vector<vector<HashKey> >::iterator i = maped_reads_back.begin();i!=maped_reads_back.end();i++){
			(*i).clear();
		}
		maped_reads_back.clear();
	}

	void clear_reads() {
		for(vector<string>::iterator i=forward.begin();i!=forward.end();i++){
			(*i).clear();
		}
		forward.clear();
		for(vector<string>::iterator i=backward.begin();i!=backward.end();i++){
			(*i).clear();
		}
		backward.clear();
	}

//
//	void map_keys_BS(Parameters parameters){
//
//		maped_reads.clear();
//
//		if(!parameters.BS_shortcut){
//			//calc keys for reverse
//			for(vector<string>::iterator i=forward.begin();i!=forward.end();i++){
//				vector<HashKey> temp_vec;
//				for(size_t j=0;j+parameters.wordsize<max_length;j=j+parameters.step_size){
//					HashKey temp;
//					//if not A,C,G,T FatsHash returns -1. -> will not be stored
//					temp.key= FatsHash((*i).substr((size_t)j,(size_t)parameters.wordsize));//,parameters);
//					if(temp.key>=0 && temp.key<=parameters.mapsize){
//						temp.pos=j;
//						temp_vec.push_back(temp);
////						int key=temp.key;
////
////						for(vector< int >::iterator k = BSlookup[key].begin();k != BSlookup[key].end();k++){
////							temp.key=(*k);
////							temp_vec.push_back(temp);
////						}
//
//					}
//				}
//				maped_reads.push_back(temp_vec);
//				temp_vec.clear();
//			}
//		}
//		maped_reads_back.clear();
//		//calc keys for reverse complement
//		for(vector<string>::iterator i=backward.begin();i!=backward.end();i++){
//			vector<HashKey> temp_vec;
//			for(size_t j=0;j+parameters.wordsize<max_length;j=j+parameters.step_size){
//				HashKey temp;
//				//if not A,C,G,T FatsHash returns -1. -> will not be stored
//				temp.key= FatsHash((*i).substr((size_t)j,(size_t)parameters.wordsize));//,parameters);
//				if(temp.key>=0 && temp.key<=parameters.mapsize){
//					temp.pos=j;
//					temp_vec.push_back(temp);
////					int key=temp.key;
////					for(vector< int >::iterator k = BSlookup[key].begin();k != BSlookup[key].end();k++){
////						temp.key=(*k);
////						temp_vec.push_back(temp);
////					}
//				}
//			}
//			maped_reads_back.push_back(temp_vec);
//			temp_vec.clear();
//		}
////		BSlookup.clear();
//	}

};

#endif /* CLASS_READS_H_ */
