#include "cuda_functions.cuh"
#include <iostream>

using std::cout;
using std::endl;

void checkCUDAError(const char *msg) {
	cudaError_t err = cudaGetLastError();
	if (cudaSuccess != err) {
		std::cerr << "Cuda error: " << msg << ": " << cudaGetErrorString(err) << "." << std::endl;
		//fprintf(stderr, "Cuda error: %s: %s.\n", msg, cudaGetErrorString(err));
		//        exit(-1);
		throw msg, cudaGetErrorString(err);
	}
}

//void DisplayProperties(cudaDeviceProp * pDeviceProp) {
//	printf("\nDevice Name \t - %s ", pDeviceProp->name);
//	printf("\n**************************************");
//	printf("\nTotal Global Memory\t\t -%d KB", pDeviceProp->totalGlobalMem / 1024);
//	printf("\nShared memory available per block \t - %d KB", pDeviceProp->sharedMemPerBlock / 1024);
//	printf("\nNumber of registers per thread block \t - %d", pDeviceProp->regsPerBlock);
//	printf("\nWarp size in threads \t - %d", pDeviceProp->warpSize);
//	printf("\nMemory Pitch \t - %d bytes", pDeviceProp->memPitch);
//	printf("\nMaximum threads per block \t - %d", pDeviceProp->maxThreadsPerBlock);
//	printf("\nMaximum Thread Dimension (block) \t - %d %d %d", pDeviceProp->maxThreadsDim[0], pDeviceProp->maxThreadsDim[1], pDeviceProp->maxThreadsDim[2]);
//	printf("\nMaximum Thread Dimension (grid) \t - %d %d %d", pDeviceProp->maxGridSize[0], pDeviceProp->maxGridSize[1], pDeviceProp->maxGridSize[2]);
//	printf("\nTotal constant memory \t - %d bytes", pDeviceProp->totalConstMem);
//	printf("\nCUDA ver \t - %d.%d", pDeviceProp->major, pDeviceProp->minor);
//	printf("\nClock rate \t - %d KHz", pDeviceProp->clockRate);
//	printf("\nTexture Alignment \t - %d bytes", pDeviceProp->textureAlignment);
//	printf("\nDevice Overlap \t - %s", pDeviceProp-> deviceOverlap ? "Allowed" : "Not Allowed");
//	printf("\nNumber of Multi processors \t - %d", pDeviceProp->multiProcessorCount);
//	printf("\n");
//
//}

//void getSpecs(int & memsize, int & mps) {
//	cudaDeviceProp deviceProp;
//	memset(&deviceProp, 0, sizeof(deviceProp));
//	if (cudaSuccess != cudaGetDeviceProperties(&deviceProp, 0)) {
//		exit(0);
//	}
//	memsize = ((deviceProp.totalGlobalMem / 1024) * 1000) - 70000000;
//	mps = deviceProp.multiProcessorCount;
//}

//int getUsableDevice() {
//	int amount = 0;
//	cudaGetDeviceCount(&amount);
//
//
//	cout << amount << endl;
//
//	int bla = 0;
//	cudaGetDevice(&bla);
//	cout << bla << endl;
//	return bla;
//}

void getDeviceProparties() {
	cudaDeviceProp deviceProp;
	int nDevCount = 0;
	cudaGetDeviceCount(&nDevCount);
	//printf("Total Device found: %d", nDevCount);
	std::cout << "Total Device found: " << nDevCount;
	for (int nDeviceIdx = 0; nDeviceIdx < nDevCount; ++nDeviceIdx) {

		memset(&deviceProp, 0, sizeof(deviceProp));
		if (cudaSuccess == cudaGetDeviceProperties(&deviceProp, nDeviceIdx)) {
			//DisplayProperties(&deviceProp);
			//numofThreads = deviceProp.maxThreadsPerBlock;
		} else
			std::cout << std::endl << cudaGetErrorString(cudaGetLastError());
			//printf("\n%s", cudaGetErrorString(cudaGetLastError()));
		return;
	}
}

int cudaDevCount(){
	int dev=0;
	cudaGetDeviceCount(&dev);
	checkCUDAError("No device");
	return dev;
}


void initCudaParameter(Parameters & parameters){

	cudaSetDevice(parameters.GPU_id);
	cudaDeviceProp deviceProp;
	memset(&deviceProp, 0, sizeof(deviceProp));
	if (cudaSuccess != cudaGetDeviceProperties(&deviceProp, parameters.GPU_id)) {
		cout << "Something is wrong with the graphics card" << endl;
		exit(0);
	}

	parameters.memsize = ((deviceProp.totalGlobalMem / 1024) * 1000) - 160000000; // TODO check solution of screen!!!
	parameters.mps = deviceProp.multiProcessorCount;
	parameters.mapsize = (int)pow(4.0, (double)parameters.wordsize);
}

void threadExit(){
	cudaThreadExit();
}
